=== modified file '__init__.py'
--- __init__.py	2006-03-22 15:21:38 +0000
+++ __init__.py	2006-03-22 22:18:19 +0000
@@ -261,13 +261,27 @@
         from branchhistory import branch_history 
         return branch_history(branch)
 
+
+class cmd_zap(bzrlib.commands.Command):
+    """\
+    Remove a checkout, if it can be done safely. <BZRTOOLS>
+
+    This command will remove a checkout without losing data.  That means
+    it only removes checkouts, 
+    """
+    takes_args = ["checkout"]
+    def run(self, checkout):
+        from zap import zap
+        return zap(checkout)
+
+
 commands = [cmd_shelve, cmd_unshelve, cmd_shelf, cmd_clean_tree,
             cmd_graph_ancestry, cmd_fetch_ghosts, cmd_patch, cmd_shell,
-            cmd_branch_history]
-
-command_decorators = []
-
-command_decorators = []
+            cmd_branch_history, cmd_zap]
+
+
+command_decorators = []
+
 
 import bzrlib.builtins
 if not hasattr(bzrlib.builtins, "cmd_push"):
@@ -318,6 +332,7 @@
     from doctest import DocTestSuite, ELLIPSIS
     from unittest import TestSuite
     import clean_tree
+    import zap
     import tests.blackbox
     import tests.shelf_tests
     result = TestSuite()
@@ -327,4 +342,5 @@
     result.addTest(tests.test_suite())
     result.addTest(TestLoader().loadTestsFromModule(tests.shelf_tests))
     result.addTest(tests.blackbox.test_suite())
+    result.addTest(zap.test_suite())
     return result

=== modified file 'clean_tree.py'
--- clean_tree.py	2006-02-28 17:17:59 +0000
+++ clean_tree.py	2006-03-22 22:18:19 +0000
@@ -60,6 +60,7 @@
                 os.unlink(path)
     if not has_deleted:
         print >> to_file, "No files deleted."
+
 def test_suite():
     from bzrlib.tests import TestCaseInTempDir
     from unittest import makeSuite

=== modified file 'errors.py'
--- errors.py	2006-03-16 14:53:00 +0000
+++ errors.py	2006-03-22 22:18:19 +0000
@@ -26,3 +26,9 @@
 
 class PatchFailed(Exception):
     """Failed applying patch!"""
+
+class NotCheckout(CommandError):
+    """Specified path is not a checkout."""
+
+class UncommittedCheckout(CommandError):
+    """This checkout contains uncommitted changes"""

=== modified file 'tests/blackbox.py'
--- tests/blackbox.py	2006-03-18 17:22:01 +0000
+++ tests/blackbox.py	2006-03-22 22:18:19 +0000
@@ -70,5 +70,11 @@
         self.runbzr('branch . my_branch')
         self.runbzr('fetch-ghosts my_branch')
 
+    def test_zap(self):
+        self.runbzr('init source')
+        self.runbzr('checkout --lightweight source checkout')
+        self.runbzr('zap checkout')
+
+
 def test_suite():
     return makeSuite(TestBzrTools)

=== added file 'zap.py'
--- zap.py	1970-01-01 00:00:00 +0000
+++ zap.py	2006-03-22 22:18:19 +0000
@@ -0,0 +1,61 @@
+from shutil import rmtree
+
+from bzrlib.errors import NoWorkingTree, NotLocalUrl, NotBranchError
+from bzrlib.delta import compare_trees
+from bzrlib.workingtree import WorkingTree
+
+from errors import NotCheckout, UncommittedCheckout
+
+
+def zap(path):
+    try:
+        wt = WorkingTree.open(path)
+    except (NoWorkingTree, NotBranchError):
+        raise NotCheckout(path)
+    tree_base = wt.bzrdir.transport.base
+    branch_base = wt.branch.bzrdir.transport.base
+    if tree_base == branch_base:
+        raise NotCheckout(path)
+    delta = compare_trees(wt.basis_tree(), wt, want_unchanged=False)
+    if delta.has_changed():
+        raise UncommittedCheckout()
+    rmtree(path)
+
+
+def test_suite():
+    import os
+    from unittest import makeSuite
+    
+    from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
+    from bzrlib.branch import BranchReferenceFormat
+    from bzrlib.tests import TestCaseInTempDir
+
+    class TestZap(TestCaseInTempDir):
+
+        def make_checkout(self):
+            wt = BzrDir.create_standalone_workingtree('source')
+            os.mkdir('checkout')
+            checkout = BzrDirMetaFormat1().initialize('checkout')
+            BranchReferenceFormat().initialize(checkout, wt.branch)
+            return checkout.create_workingtree()
+
+        def test_is_checkout(self):
+            self.assertRaises(NotCheckout, zap, '.')
+            wt = BzrDir.create_standalone_workingtree('.')
+            self.assertRaises(NotCheckout, zap, '.')
+
+        def test_zap_works(self):
+            self.make_checkout()
+            self.assertIs(True, os.path.exists('checkout'))
+            zap('checkout')
+            self.assertIs(False, os.path.exists('checkout'))
+
+        def test_checks_modified(self):
+            checkout = self.make_checkout()
+            os.mkdir('checkout/foo')
+            checkout.add('foo')
+            self.assertRaises(UncommittedCheckout, zap, 'checkout')
+            checkout.commit('commit changes to branch')
+            zap('checkout')
+
+    return makeSuite(TestZap)

